module Hermes
  class HousingRequestInitializer
    include Hermes::Concerns::CommonFieldsConcern
    RANDOM_LIST_CHAR = '012345678901234567890123456789012345678901234567890123abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    
    field :unique_number, type: String
    field :hashed_pin_code, type: String
    field :pin_code_created_at, type: DateTime
    field :email, type: String
    field :xml_res, type: String
    
    def hash_string(str)
      Digest::SHA2.new(512).hexdigest(str)
    end
    
    def format_mail_body(pin_code, bouchon_mail)
      "Bonjour #{bouchon_mail},

vous recevez ce mél suite à une tentative de récupération de votre demande sur la base de son numéro unique.
Afin de confirmer cette récupération pour que votre demande soit dupliquée sur notre site MaSolutionLogement, veuillez saisir le clé de sécurité dans la zone prévue à cet effet.

Clé de sécurité : #{pin_code}

Attention, cette clé de sécurité est valable une heure. Passé ce délai il sera nécessaire de recommencer votre démarche.
Si vous n'êtes pas à l'origine de cette demande, veuillez ne pas prendre en considération ce mél.

Cordialement

Toute l'équipe Ma Solution Logement

ps : ceci est un message automatique, il est inutile de répondre à ce mél.
"
    end
    
    def call_sne
      #todo: faire toute la requete à partir du numéro unique jusqu'a récuperer
      path = [Hermes::Engine.root, %w[spec xml_bouchon], "DEM_" + unique_number + ".xml"].flatten.join('/').to_s
      self.xml_res = File.read(path)
      parsed_xml = Hash.from_xml(xml_res.gsub("\n", ""))
      
      bouchon_mail = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["demandeur"]["mel"]
      return {email: "recette.hermes@actionlogement.fr", err: nil, bouchon_mail: bouchon_mail}
    end
    
    def send_pin_code_on_mail
      sne_response = call_sne
      return sne_response[:err] if sne_response[:err]
      return false unless sne_response[:email]
      
      random_list_array = RANDOM_LIST_CHAR.split(//)
      pin_code = ''
      6.times do |i|
        pin_code += random_list_array[(SecureRandom.random_number * random_list_array.length).floor]
      end
      self.hashed_pin_code = hash_string(pin_code)
      self.pin_code_created_at = DateTime.current
      mail = Agent.bulk_mail(
          sne_response[:email],
          'securite@masolutionlogement.fr', #
          'Récupération demande SNE',
          format_mail_body(pin_code, sne_response[:bouchon_mail])
      )
      return false unless mail.deliver_later
      save
      pin_code
    end
    
    def check_pin_code(user_input)
      return false unless hashed_pin_code && pin_code_created_at
      
      hash_string(user_input) == hashed_pin_code && (Time.current - pin_code_created_at) < 1.hours
    end
    
    def hydrate_housing_request_from_sne(user_id)
      parsed_xml = parsed_xml = Hash.from_xml(xml_res.gsub("\n", ""))
      main_applicant = Hermes::Applicant.create(user_id: user_id)
      
      
      housing_request = Hermes::HousingRequest.create(user_id: user_id, main_applicant: main_applicant.id)
      
      nud = Hermes::Nud.create(unique_number: unique_number, housing_request_id: housing_request.id)
      housing_request.nud_ids.push(nud.id)
      main_applicant.update(applicant_reversed_map(parsed_xml))
      housing_request.update(housing_request_reversed_map(parsed_xml))
      main_applicant.save()
      housing_request.save()
      return housing_request.reload
    end
    
    def applicant_map
    
    end
    
    def housing_request_reversed_map(xml)
      demande_logement = xml["interfaceNuu"]["demande"]["demandeLogement"] #["personnePhysique"]["demandeur"]["mel"]
      personnePhysique = demande_logement["personnePhysique"]
      demandeur = personnePhysique["demandeur"]
      
      attributes = {
          
          forcing_duplicate: demande_logement["forcerDoublon"],
          electronical_renewal: demande_logement["renouvellementElec"],
          widen_request: demande_logement["logementRecherche"]["demandeElargie"],
          work_contract_type: [
              demandeur["situationProfessionnelle"]["typeContratTravail"]["code"]
          ],
          motivation1: demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch(0, {})["code"],
          motivation2: demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch(1, {})["code"],
          motivation3: demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch(2, {})["code"],
          seeking_first_floor: demande_logement["logementRecherche"]["rezDeChaussee"],
          seeking_elevator: demande_logement["logementRecherche"]["sansAscenseur"],
          seeked_max_rent: demande_logement["logementRecherche"]["montantLoyer"].to_i,
          precisions: demande_logement["precision"],
          seeked_housing_categories: [
              demande_logement["logementRecherche"]["categorieLogementRecherche"]["code"]
          ],
          is_house_owner: personnePhysique["logementActuel"]["proprietaire"],
          als_id: demande_logement["refInterne"],
          requester_firstname: demandeur["nom"],
          requester_lastname: demandeur["prenom"],
          requester_email: demandeur["mel"],
          created_at: demande_logement["dateCreationDemande"],
        
        
      }
      
      
      attributes[:seeked_types] = []
      typeLogement = demande_logement["logementRecherche"]["listeTypeLogement"]["typeLogement"]
      typeLogement = typeLogement.is_a?(Array) ? typeLogement : [typeLogement]
      typeLogement&.each do |type|
        attributes[:seeked_types].push(type["code"].to_s)
      end
      
      attributes[:dependents] = []
      demandeur["listeRessourceRecuePersACharge"] = demandeur["listeRessourceRecuePersACharge"].is_a?(Array) ? demandeur["listeRessourceRecuePersACharge"] : [demandeur["listeRessourceRecuePersACharge"]]
      demandeur["listeRessourceRecuePersACharge"]&.each_with_index do |persACharge, i|
        if (persACharge && persACharge["listeRessourceRecue"])
          persACharge&.fetch("listeRessourceRecue", [])&.each do |ressourceRecue|
            attributes[:dependents][i]["monthly_incomes"][ressourceRecue["detailRessource"]["ressource"]['code'].to_s] = ressourceRecue["detailRessource"]["montant"].to_i
          end
        end
      end
      demandeur["listeRessourceVerseePersACharge"] = demandeur["listeRessourceVerseePersACharge"].is_a?(Array) ? demandeur["listeRessourceVerseePersACharge"] : [demandeur["listeRessourceVerseePersACharge"]]
      demandeur["listeRessourceVerseePersACharge"]&.each_with_index do |persACharge, i|
        if (persACharge && persACharge["listeRessourceVersee"])
          persACharge.fetch("listeRessourceVersee", [])&.each do |ressourceVersee|
            attributes[:dependents][i]["monthly_charges"][ressourceVersee["detailRessource"]["ressource"]['code'].to_s] = ressourceVersee["detailRessource"]["montant"].to_i
          end
        end
      
      end
      
      attributes[:postal_details] = {}
      adressePostale = demandeur["adressePostale"] || {}
      attributes[:postal_details]['etranger'] = adressePostale["etranger"]
      attributes[:postal_details]['appartment_number'] = adressePostale["appartement"]
      attributes[:postal_details]['appartment_floor'] = adressePostale["etage"]
      attributes[:postal_details]['appartment_staircase'] = adressePostale["escalier"]
      attributes[:postal_details]['appartment_building'] = adressePostale["batiment"]
      attributes[:postal_details]['address_add_address'] = adressePostale["compAdresse"]
      attributes[:postal_details]['address_number'] = adressePostale["numero"]
      attributes[:postal_details]['address_street'] = adressePostale["voie"]
      attributes[:postal_details]['appartment_address_place'] = adressePostale["lieuDit"]
      attributes[:postal_details]['address_code'] = adressePostale["codePostal"]
      attributes[:postal_details]['address_insee_code'] = adressePostale["commune"]&.fetch("code", nil)

      attributes[:seeked_localisation_list] = []
      localisationSouhaite = demande_logement["logementRecherche"]["listeLocalisationSouhaite"]["localisationSouhaite"]
      localisationSouhaite = localisationSouhaite.is_a?(Array) ? localisationSouhaite : [localisationSouhaite]
      localisationSouhaite.each_with_index do |loc,i|
        if( loc)
          res = {}
          res["code"] = loc["commune"]["code"]
          res["codesPostaux"] = [loc["codePostal"]]
          res["seeked_epci_first_quartil"] = loc["seuilEPCIPremierQuartile"]
          attributes[:seeked_localisation_list].push res
        end
      end
      
      return attributes
    end
    
    def applicant_reversed_map(xml)
      demande_logement = xml["interfaceNuu"]["demande"]["demandeLogement"] #["personnePhysique"]["demandeur"]["mel"]
      personnePhysique = demande_logement["personnePhysique"]
      demandeur = personnePhysique["demandeur"]
      
      attributes = {
          nationality: demandeur["nationalite"]["code"],
          civility: demandeur["civilite"]["code"],
          birth_date: demandeur["dateNaissance"],
          has_other_employers: demandeur["situationProfessionnelle"]["multiEmployeur"],
          year_minus_one: demandeur["revenuFiscal"]["anneeMoins1"],
          amount_year_minus_one: demandeur["revenuFiscal"]["montantMoins1"].to_i,
          year_minus_two: demandeur["revenuFiscal"]["anneeMoins2"],
          amount_year_minus_two: demandeur["revenuFiscal"]["montantMoins2"].to_i,
          family_situation: demandeur["situationFamiliale"]["code"],
      }
      attributes[:actual_housing_details] = {
      
      }
      adresseLogement = personnePhysique["adresseLogement"] || {}
      logementActuel = personnePhysique["logementActuel"] || {}
      attributes[:actual_housing_details]['etranger'] = adresseLogement["etranger"]
      attributes[:actual_housing_details]['appartment_number'] = adresseLogement["appartement"]
      attributes[:actual_housing_details]['appartment_floor'] = adresseLogement["etage"]
      attributes[:actual_housing_details]['appartment_staircase'] = adresseLogement["escalier"]
      attributes[:actual_housing_details]['appartment_building'] = adresseLogement["batiment"]
      attributes[:actual_housing_details]['address_add_address'] = adresseLogement["compAdresse"]
      attributes[:actual_housing_details]['appartment_address_place'] = adresseLogement["lieuDit"]
      attributes[:actual_housing_details]['address_number'] = adresseLogement["numero"]
      attributes[:actual_housing_details]['way'] = adresseLogement["voie"]
      attributes[:actual_housing_details]['address_code'] = adresseLogement["codePostal"]
      attributes[:actual_housing_details]['address_insee_code'] = adresseLogement["commune"]&.fetch("code", nil)
      attributes[:actual_housing_details]['housing_typology'] = logementActuel["listeModeleLogement"]&.fetch("modeleLogement", nil)&.fetch("code", nil)
      attributes[:actual_housing_details]['housing_rent_amount'] = logementActuel["montantMensuelLoyer"].to_i
      attributes[:actual_housing_details]['housing_apl'] = logementActuel["montantMensuelAPL"].to_i
      attributes[:actual_housing_details]['housing_resident_nb'] = logementActuel["nombreHabitant"]
      attributes[:actual_housing_details]['housing_surface'] = logementActuel["surface"]
      attributes[:actual_housing_details]['is_house_owner'] = logementActuel["proprietaire"]
      attributes[:actual_housing_details]['etranger'] = logementActuel["etranger"]
      
      attributes[:monthly_incomes] = {}
      demandeur['listeRessourceRecue'] = demandeur['listeRessourceRecue'].is_a?(Array) ? demandeur['listeRessourceRecue'] : [demandeur['listeRessourceRecue']]
      demandeur['listeRessourceRecue']&.each do |ressource|
     
        if ressource
          detailRessource = ressource["detailRessource"].is_a?(Array) ? ressource["detailRessource"] : [ressource["detailRessource"]]
          detailRessource.each do |detail|
            attributes[:monthly_incomes][detail["ressource"]["code"].to_s] = detail["montant"].to_i
          end
        
        end
      end
      attributes[:monthly_charges] = {}
      
      demandeur['listeRessourceVersee'] = demandeur['listeRessourceVersee'].is_a?(Array) ? demandeur['listeRessourceVersee'] : [demandeur['listeRessourceVersee']]
      demandeur['listeRessourceVersee']&.each do |ressource|
        if ressource
          detailRessource = ressource["detailRessource"].is_a?(Array) ? ressource["detailRessource"] : [ressource["detailRessource"]]
          detailRessource.each do |detail|
            attributes[:monthly_charges][detail["ressource"]["code"].to_s] = detail["montant"].to_i
          end
        end
      end
      
   
      return attributes
    end
  end

end