# frozen_string_literal: true

module Hermes
  # HousingApplication serializer
  class HousingApplicationSerializer < Hermes::ApplicationSerializer
    attributes %i[
      applicated_at
      applicant_id
      housing_request_id
      housing_offer_id
      max_date
      epci
      effort_rate
      profile_epci_score
      status
      status_updated_at
      score_with_position
      eligible_position
      applicated_position
      effort_rate
      score_id
      housing_request
    ]

    def embedded_applicant?
      instance_options[:embedded_applicant] == true
    end

    attribute :applicant, if: :embedded_applicant?

    type 'housing_applications'

    def applicant_id
      object.applicant_id&.to_s
    end

    def applicant
      return unless object.applicant

      Hermes::ApplicantSerializer.new(
        object.applicant,
        scope: scope,
        root: false
      )
    end

    def housing_request_id
      object.housing_request_id&.to_s
    end

    def housing_offer_id
      object.housing_offer_id&.to_s
    end

    def score
      @score ||= Hermes::Score.find_by(
        applicant_id: object.housing_request&.main_applicant_id,
        housing_offer_id: housing_offer_id
      )
    end

    def profile_epci_score
      score&.profile_epci_score
    end

    def score_with_position
      score&.score_with_position
    end

    def eligible_position
      score&.eligible_position
    end

    def applicated_position
      score&.applicated_position
    end

    def effort_rate
      score&.effort_rate
    end

    def score_id
      score&.id.to_s
    end

    def housing_request
      return unless instance_options[:join_housing_request]

      Hermes::HousingRequestSerializer.new(
        object.housing_request,
        scope: scope,
        root: false
      )
    end
  end
end
